from security.token_manager import TokenManager
import network_constants
from db.entities.layer import Layer
from db.repository import Repository

from typing import List, Dict
import requests
import json

def _fetch_layers(use_case: str) -> List[Layer]:
    jwt = TokenManager.getInstance().getToken()

    url = f'https://{network_constants.BUSINESS_LOGIC_HOSTNAME}:{network_constants.BUSINESS_LOGIC_REST_PORT}/api/use-cases/{use_case}/layers'

    response = requests.get(
            url,
            verify = False, 
            proxies = { "http":None, "https":None },
            headers = {"Authorization": f"Bearer {jwt}"}
    )

    if response.status_code != 200:
        raise ConnectionError(f"Could not fetch layers for {use_case} from business-logic microservice, statuscode: {response.status_code}!")

    data = json.loads(response.text)

    layers = []

    for row in data:
        layers.append(Layer.from_business_logic_dict(row))
    
    return layers

def _fetch_nodes(use_case: str, layer: Layer) -> List[Dict]:
    jwt = TokenManager.getInstance().getToken()

    url = f'https://{network_constants.SEMANTIC_LINKING_HOSTNAME}:{network_constants.SEMANTIC_LINKING_REST_PORT}/api/use-cases/{use_case}/layers/{layer.layer_name}/nodes'

    response = requests.get(
            url,
            verify = False, 
            proxies = { "http":None, "https":None },
            headers = {"Authorization": f"Bearer {jwt}"}
    )

    if response.status_code != 200:
        raise ConnectionError(f"Could not fetch nodes for {use_case}/{layer.layer_name} from semantic-linking microservice, statuscode: {response.status_code}!")

    data = json.loads(response.text)

    nodes = []

    for row in data:
        nodes.append(row)
    
    return nodes

def _fetch_use_cases() -> List[str]:
    jwt = TokenManager.getInstance().getToken()

    url = f'https://{network_constants.BUSINESS_LOGIC_HOSTNAME}:{network_constants.BUSINESS_LOGIC_REST_PORT}/api/use-cases'

    response = requests.get(
            url,
            verify = False, 
            proxies = { "http":None, "https":None },
            headers = {"Authorization": f"Bearer {jwt}"}
    )

    if response.status_code != 200:
        raise ConnectionError(f"Could not fetch use-cases from business-logic microservice, statuscode: {response.status_code}!")

    data = json.loads(response.text)

    return [row["name"] for row in data]

def fetch_nodes_from_semantic_linking():
    # prune DB 
    repository = Repository()
    repository.delete_all_layers()
    repository.delete_all_nodes()

    use_cases = _fetch_use_cases()
    for use_case in use_cases:
        print(f"Fetching for use-case {use_case}")
        layers = _fetch_layers(use_case)

        for layer in layers:
            try:
                print(f"Fetching nodes for layer {layer.layer_name}")
                # check if layer already exists in DB, add it if not
                reference_layer = repository.get_layer_for_use_case(layer.layer_name, use_case)
                if reference_layer == None:
                    repository.add_layer(layer)

                nodes = _fetch_nodes(use_case, layer)

                for node in nodes:
                    repository.add_layer_node(node)
            except ConnectionError as e:
                print(str(e))
                continue
            