#global imports
from db.entities.layer_adapter import LayerAdapter
from db.schema_repository import SchemaRepository
from services.layer_adapter_service import LayerAdapterService

import json
from flask import Response, request

schema_repository = SchemaRepository()

def all():
    return [schema.to_serializable_dict() for schema in schema_repository.all()]

def get_for_use_case(use_case:str):
    schema = schema_repository.get_for_use_case(use_case)

    if schema != None:
        return Response(status=200, response=json.dumps(schema.to_serializable_dict()))
    
    return Response(status=404, response=f"Schema {use_case} does not exist")

def delete_for_use_case(use_case:str):
    schema = schema_repository.get_for_use_case(use_case)

    if schema == None:
        return Response(status=404, response=f"Schema {use_case} does not exist")
    
    schema_repository.delete_for_use_case(use_case)

    return Response(status=200)

def add_mapping(use_case:str):
    data = request.json

    if "internal" not in data or "external" not in data:
        return Response(status=400, response=f"Field missing! Fields required: (internal, external)")
    
    schema = schema_repository.get_for_use_case(use_case)

    if schema == None:
        print("schema not there, creating it...")
        schema = schema_repository.put(use_case)
    
    schema.add_mapping(data["internal"], data["external"])
    schema_repository.update(schema)

    return Response(status=200)